@@ -110,6 +110,19 @@ test_that("Order splitting working for multiclass probability", {
   expect_true(all(rf$forest$is.ordered))
 })
 
+test_that("Order splitting working with alternative interface", {
+  n <- 20
+  dt <- data.frame(x = sample(c("A", "B", "C", "D"), n, replace = TRUE), 
+                   y = factor(sample(c("A", "B", "C", "D"), n, replace = TRUE)),
+                   stringsAsFactors = FALSE)
+  
+  rf <- ranger(dependent.variable.name = "y", data = dt, num.trees = 5, respect.unordered.factors = 'order')
+  expect_true(all(rf$forest$is.ordered))
+  
+  rf <- ranger(dependent.variable.name = "y", data = dt, num.trees = 5, respect.unordered.factors = 'order', probability = TRUE)
+  expect_true(all(rf$forest$is.ordered))
+})
+
 test_that("Unordered splitting working for survival", {
   rf <- ranger(Surv(time, status) ~ ., veteran, num.trees = 5, min.node.size = 50, respect.unordered.factors = 'partition')
   expect_true(any(!rf$forest$is.ordered))
@@ -120,6 +133,11 @@ test_that("Order splitting working for survival", {
   expect_true(all(rf$forest$is.ordered))
 })
 
+test_that("Order splitting working for survival with alternative interface", {
+  rf <- ranger(dependent.variable.name = "time", status.variable.name = "status", data = veteran, num.trees = 5, min.node.size = 50, respect.unordered.factors = 'order')
+  expect_true(all(rf$forest$is.ordered))
+})
+
 test_that("Error if too many factors in 'partition' mode", {
   n <- 100
   dt <- data.frame(x = factor(1:100, ordered = FALSE),  
@@ -210,3 +228,10 @@ test_that("No error if new levels in predict, 2 columns", {
   rf.order <- ranger(y ~ ., data = train, num.trees = 5, respect.unordered.factors = 'order')
   expect_silent(predict(rf.order, test))
 })
+
+test_that("No error if NA factor levels and order", {
+  df <- data.frame(x = addNA(factor(c("a", "a", NA, NA, "b", "b"))),
+                   y = c(1, 2, 3, 4, 5, 6))
+  expect_silent(ranger(dependent.variable.name = "y", data = df, 
+                respect.unordered.factors = "order"))
+})
